;
;
;	Soler & PIR Guide lamp  0.5.4Bd
;			LED: SinkDrive
;			SolarCell: ETM250-1.5V
;			Battery: NiMH 1.2V 1cell
;
;		The 3rd experimental version
;			based 0.4.3
;			Software PWM Drive x RGB
;				Clock up 31KHz to 250KHz
;				PWM refresh by timer0
;				Battery voltage 6color display
;				1MHz clock, Half TMR0, 16 level
;			Power tuning -- Standby power reduce
;				LED port output mode only
;				Tri-state is current leak
;				1470uA->230uA / 1.285V
;			Power tuning2 -- Standby power reduce
;				BOD disable
;				200uA->30uA / 1.39V
;			Customize1 -- lamp color table
;				Table: Battery voltage -> Color#
;			First V0.5 -- Di -> FET control 0.5.0<2006.06.07-09>
;				AN0 -> Multi use
;					0/16sec  : AN0->input, measurement solar cell vol
;					1~7/16sec: AN1->output, gate control
;				AN1 -> Battery voltage
;			Customize2 -- lamp color table 0.5.1<2006.06.21-22>
;				Table: Color# -> R%:G%:B%, Pink & Orange append
;				PWM resolution up, 16level(4bit) -> 32level(5bit)
;			Bug fix -- 0.5.1a<2006.07.06>
;				CMCON0 initalyze miss, BANKSEL use
;				PGID: Program ID ROM marking
;			Bug fix -- 0.5.1b<2006.07.27>
;				Night thretold: deep
;			Bug fix -- 0.5.1c<2006.08.15>
;				Night thretold: deep+
;			Bug fix -- 0.5.1d<2006.11.20>
;				Night thretold: deep-
;			PWM phase shift -- 0.5.2d<2006.12.20>
;				Night thretold: deep-
;			Power stable 0.5.3d<2006.12.21>
;				PWRTE OFF->ON
;				Night thretold: deep-
;			Customize3 -- lamp color table 0.5.4B<2007.01.03>
;				White -> LightBlue -> Green -> Yellow -> Red -> Blue
;				Night thretold: deep-
;
;
		INCLUDE	P12F683.INC
		ERRORLEVEL  -302,-306
		__CONFIG    _CP_OFF & _PWRTE_ON & _WDT_ON & _INTOSCIO & _MCLRE_OFF & _BOD_OFF

;=======================
;       I/O Port (Review initial port settings)
;=======================
;IOSCELVOL	EQU		0			; AI:Solar cell voltage
IOSCGATE	EQU		0			; DO,AI:Solar cell gate / voltage
IOBATTVOL	EQU		1			; AI:Battery voltage
IOLEDRED	EQU		2			; DO:Main LED RED
IOPIRSIG	EQU		3			; DI:PIR Signal
IOLEDGREEN	EQU		4			; DO:Main LED GREEN
IOLEDBLUE	EQU		5			; DO:Main LED BLUE

;=======================
;       Parameter
;=======================
LAMPPERIOD		EQU		D'63'		; Main LED PWM Period 128Hz by 32k clock(1unit Fosc/4)
LAMPDARK		EQU		D'0'		; Main LED duty rate, off 0% (sink drive:Vdd-LED-PIC-GND)

LAMPHIDUTY1		EQU		D'15'		; Main LED duty rate, night 50%
LAMPLODUTY1		EQU		D'1'		; Main LED duty rate, night 3%
LAMPHIDUTY2		EQU		D'31'		; Main LED duty rate, PIR sense 100%
LAMPLODUTY2		EQU		D'0'		; Main LED duty rate, PIR sense 0%

LAMPONTIME1		EQU		D'14400'	; Main LED on sec, sunset 4H
LAMPONTIME1H	EQU		(LAMPONTIME1 >> 8) & 0xff
LAMPONTIME1L	EQU		LAMPONTIME1 & 0xff
LAMPONTIME2		EQU		D'60'		; Main LED on sec, PIR sense 1Min
LAMPONTIME2H	EQU		(LAMPONTIME2 >> 8) & 0xff
LAMPONTIME2L	EQU		LAMPONTIME2 & 0xff

LAMPFOTIME1		EQU		D'1'		; Slow flush on sec,  night
LAMPFFTIME1		EQU		D'1'		; Slow flush off sec, night
LAMPFOTIME2		EQU		D'1'		; Slow flush on sec,  PIR sense
LAMPFFTIME2		EQU		D'1'		; Slow flush off sec, PIR sense

;LEDALLON		EQU		B'00110100'	;	/ source drive
;LEDALLOFF		EQU		B'00000000'	;
LEDALLON		EQU		B'00000000'	;	/ sink drive
LEDALLOFF		EQU		B'00110100'	;

; ** @13.95mV=Vdd<3.57V>/256 **

; Solar cell <Vsc>
;BRIGHTDAY		EQU		D'108'		; 1.5V SC-4050
;BRIGHTNIGHT	EQU		D'93'		; 1.3V SC-4050
;BRIGHTDAY		EQU		D'43'		; 0.6V ETM250-1.5V
;BRIGHTNIGHT	EQU		D'14'		; 0.2V ETM250-1.5V
;BRIGHTDAY		EQU		D'72'		; 1.0V ETM250-1.5V  2006/3/2
;BRIGHTNIGHT	EQU		D'22'		; 0.3V ETM250-1.5V  2006/3/2
;BRIGHTDAY		EQU		D'43'		; 0.6V Solar charger 1/4
;BRIGHTNIGHT	EQU		D'7'		; 0.1V Solar charger 1/4
BRIGHTDAY		EQU		D'72'		; 1.0V ETM250-1.5V  2006/11/20
BRIGHTNIGHT		EQU		D'46'		; 0.64V ETM250-1.5V  2006/11/20

; Battery <Vbatt>
BATTHIGHU		EQU		D'100'		; 1.40V NiMH
BATTHIGHD		EQU		D'93'		; 1.30V NiMH
BATTLOWU		EQU		D'92'		; 1.28V NiMH
BATTLOWD		EQU		D'86'		; 1.20V NiMH
;BATTHIGHU		EQU		D'201'		; 2.8V GoldCap
;BATTHIGHD		EQU		D'165'		; 2.3V GoldCap
;BATTLOWU		EQU		D'72'		; 1.0V GoldCap
;BATTLOWD		EQU		D'7'		; 0.1V GoldCap

TMR1V		EQU		D'819'		; 100ms, Fosc(32k)/4 unit
TMR1VH		EQU		(-TMR1V >> 8) & H'FF'
TMR1VL		EQU		-TMR1V & H'FF'

;=======================
;       Work area
;=======================
USERWORK	EQU		0x20		; work area top
AN0VAL		EQU		0x20		; Analog value
AN1VAL		EQU		0x21
ISR_W		EQU		0x22		; House keep
ISR_S		EQU		0x23		; House keep
ANXVAL		EQU		0x24		; Analog work value
ANYCMD		EQU		0x25		; Analog work command
ANYVALS		EQU		0x26		; span
ANYVALH		EQU		0x27		; high limit value
ANYVALL		EQU		0x28		; low limit value
ANYVAL1		EQU		0x29		; 1st value
ANYVAL2		EQU		0x2a		; 2nd value
ANYVAL3		EQU		0x2b		; 3rd value
WAITCTR		EQU		0x2c		; Pause subroutine
WAITCTR2	EQU		0x2d		; Pause subroutine

PFLAG		EQU		0x30		; Power flags
								;   Bit0:Power down
								;   Bit1:Lamp on/off (Real)
								;   Bit2:On timer
								;   Bit3:Night/Day
								;   Bit4:Over load

LAMPFLAG		EQU	0x31		; Control flags
								;   Bit3:Slow flush direction 1:low->hi
								;   Bit5:Lamp on/off (request)
LAMPDUTYSET		EQU	0x32		; Target LED bright
LAMPHIDUTY		EQU	0x33		; upper LED bright
LAMPLODUTY		EQU	0x34		; lower LED bright
LAMPTIMEFL		EQU	0x35		; Slow flush timer
LAMPFOTIME		EQU	0x36		; Flash on time
LAMPFFTIME		EQU	0x37		; Flash off time
LAMPTIMEON1H	EQU	0x38		; Light on timer for night
LAMPTIMEON1L	EQU	0x39
LAMPTIMEON2H	EQU	0x3a		; Light on timer for PIR
LAMPTIMEON2L	EQU	0x3b
LAMPWFLAG		EQU	0x3c		; Zero check flags
								;   bit1:timer1<>0
								;   bit2:timer2<>0
								;   bit3:flush<>0
LAMPWFLAGB		EQU	0x3d		; before LAMPWFLAG
MAIN_CTR		EQU	0x3e		; Slow LED on counter

PWM_DUTY		EQU	0x40		; Live duty rate
PWM_COLOR		EQU	0x41		; Live color#
PWM_CTR			EQU	0x42		; Refresh counter
PWM_COLOR_BYTE	EQU	0x43		; xxRRGGBB, 00=0% 01=25% 10=50% 11=100%
PWM_WORK		EQU	0x44		; Temporary
PWM_WORK2		EQU	0x45		; Temporary

SCAN_STATE		EQU	0x46		; 16sec cycle counter
VBATT			EQU	0x47		; Battery voltate / on load

PWM_TBL			EQU	0x50		; must be 32byte align


;===============================
;         Vector area
;===============================
		ORG	0x0000

		GOTO  	 START

		ORG	0x0004
		GOTO  	 ISR


;===============================
;     Table area (5-255word)
;===============================
		ORG	0x005
MAKECOLOR_BYTE
		ANDLW	B'00111111'		; xxRRGGBB
		ADDWF	PCL,F
		RETLW	B'00000000'		;  0:dummy
		RETLW	B'00000011'		;  1:Blue   2007.1.3
		RETLW	B'00110011'		;  2:Purple
		RETLW	B'00110001'		;  3:Pink
		RETLW	B'00110000'		;  4:Red
		RETLW	B'00111000'		;  5:Orange
		RETLW	B'00111100'		;  6:Yellow
		RETLW	B'00011100'		;  7:Lime
		RETLW	B'00001100'		;  8:Green
		RETLW	B'00001101'		;  9:BlueGreen
		RETLW	B'00001111'		; 10:LightBlue
		RETLW	B'00101111'		; 11:LightSkyBlue
		RETLW	B'00111111'		; 12:White
		RETLW	B'00110111'		; 13:LightPurple


VOLCOLOR
		ADDWF	PCL,F			; Under value
		RETLW	D'0'			;  0:0.000V (dummy entry)
		RETLW	D'86'			;  1:1.200V
		RETLW	D'87'			;  2:1.214V  2007.1.3
		RETLW	D'88'			;  3:1.228V
		RETLW	D'89'			;  4:1.242V
		RETLW	D'90'			;  5:1.256V
		RETLW	D'91'			;  6:1.269V
		RETLW	D'92'			;  7:1.283V
		RETLW	D'93'			;  8:1.297V
		RETLW	D'94'			;  9:1.311V
		RETLW	D'95'			; 10:1.325V
		RETLW	D'96'			; 11:1.339V
		RETLW	D'97'			; 12:1.353V
		RETLW	D'100'			; 13:1.395V

MAKEPATTRN_RED
		ANDLW	B'00011111'
		ADDWF	PCL,F
		BSF		PWM_TBL+D'0',IOLEDRED
		BSF		PWM_TBL+D'16',IOLEDRED
		BSF		PWM_TBL+D'8',IOLEDRED
		BSF		PWM_TBL+D'24',IOLEDRED
		BSF		PWM_TBL+D'4',IOLEDRED
		BSF		PWM_TBL+D'20',IOLEDRED
		BSF		PWM_TBL+D'12',IOLEDRED
		BSF		PWM_TBL+D'28',IOLEDRED
		BSF		PWM_TBL+D'2',IOLEDRED
		BSF		PWM_TBL+D'18',IOLEDRED
		BSF		PWM_TBL+D'10',IOLEDRED
		BSF		PWM_TBL+D'26',IOLEDRED
		BSF		PWM_TBL+D'6',IOLEDRED
		BSF		PWM_TBL+D'22',IOLEDRED
		BSF		PWM_TBL+D'14',IOLEDRED
		BSF		PWM_TBL+D'30',IOLEDRED
		BSF		PWM_TBL+D'1',IOLEDRED
		BSF		PWM_TBL+D'17',IOLEDRED
		BSF		PWM_TBL+D'9',IOLEDRED
		BSF		PWM_TBL+D'25',IOLEDRED
		BSF		PWM_TBL+D'5',IOLEDRED
		BSF		PWM_TBL+D'21',IOLEDRED
		BSF		PWM_TBL+D'13',IOLEDRED
		BSF		PWM_TBL+D'29',IOLEDRED
		BSF		PWM_TBL+D'3',IOLEDRED
		BSF		PWM_TBL+D'19',IOLEDRED
		BSF		PWM_TBL+D'11',IOLEDRED
		BSF		PWM_TBL+D'27',IOLEDRED
		BSF		PWM_TBL+D'7',IOLEDRED
		BSF		PWM_TBL+D'23',IOLEDRED
		BSF		PWM_TBL+D'15',IOLEDRED
		BSF		PWM_TBL+D'31',IOLEDRED
		XORLW	B'00011111'
		ADDWF	PCL,F
		BCF		PWM_TBL+D'31',IOLEDRED
		BCF		PWM_TBL+D'15',IOLEDRED
		BCF		PWM_TBL+D'23',IOLEDRED
		BCF		PWM_TBL+D'7',IOLEDRED
		BCF		PWM_TBL+D'27',IOLEDRED
		BCF		PWM_TBL+D'11',IOLEDRED
		BCF		PWM_TBL+D'19',IOLEDRED
		BCF		PWM_TBL+D'3',IOLEDRED
		BCF		PWM_TBL+D'29',IOLEDRED
		BCF		PWM_TBL+D'13',IOLEDRED
		BCF		PWM_TBL+D'21',IOLEDRED
		BCF		PWM_TBL+D'5',IOLEDRED
		BCF		PWM_TBL+D'25',IOLEDRED
		BCF		PWM_TBL+D'9',IOLEDRED
		BCF		PWM_TBL+D'17',IOLEDRED
		BCF		PWM_TBL+D'1',IOLEDRED
		BCF		PWM_TBL+D'30',IOLEDRED
		BCF		PWM_TBL+D'14',IOLEDRED
		BCF		PWM_TBL+D'22',IOLEDRED
		BCF		PWM_TBL+D'6',IOLEDRED
		BCF		PWM_TBL+D'26',IOLEDRED
		BCF		PWM_TBL+D'10',IOLEDRED
		BCF		PWM_TBL+D'18',IOLEDRED
		BCF		PWM_TBL+D'2',IOLEDRED
		BCF		PWM_TBL+D'28',IOLEDRED
		BCF		PWM_TBL+D'12',IOLEDRED
		BCF		PWM_TBL+D'20',IOLEDRED
		BCF		PWM_TBL+D'4',IOLEDRED
		BCF		PWM_TBL+D'24',IOLEDRED
		BCF		PWM_TBL+D'8',IOLEDRED
		BCF		PWM_TBL+D'16',IOLEDRED
;		BCF		PWM_TBL+D'0',IOLEDRED
		RETURN
MAKEPATTRN_GREEN
		ANDLW	B'00011111'
		ADDWF	PCL,F
		BSF		PWM_TBL+D'31',IOLEDGREEN
		BSF		PWM_TBL+D'0',IOLEDGREEN
		BSF		PWM_TBL+D'16',IOLEDGREEN
		BSF		PWM_TBL+D'8',IOLEDGREEN
		BSF		PWM_TBL+D'24',IOLEDGREEN
		BSF		PWM_TBL+D'4',IOLEDGREEN
		BSF		PWM_TBL+D'20',IOLEDGREEN
		BSF		PWM_TBL+D'12',IOLEDGREEN
		BSF		PWM_TBL+D'28',IOLEDGREEN
		BSF		PWM_TBL+D'2',IOLEDGREEN
		BSF		PWM_TBL+D'18',IOLEDGREEN
		BSF		PWM_TBL+D'10',IOLEDGREEN
		BSF		PWM_TBL+D'26',IOLEDGREEN
		BSF		PWM_TBL+D'6',IOLEDGREEN
		BSF		PWM_TBL+D'22',IOLEDGREEN
		BSF		PWM_TBL+D'14',IOLEDGREEN
		BSF		PWM_TBL+D'30',IOLEDGREEN
		BSF		PWM_TBL+D'1',IOLEDGREEN
		BSF		PWM_TBL+D'17',IOLEDGREEN
		BSF		PWM_TBL+D'9',IOLEDGREEN
		BSF		PWM_TBL+D'25',IOLEDGREEN
		BSF		PWM_TBL+D'5',IOLEDGREEN
		BSF		PWM_TBL+D'21',IOLEDGREEN
		BSF		PWM_TBL+D'13',IOLEDGREEN
		BSF		PWM_TBL+D'29',IOLEDGREEN
		BSF		PWM_TBL+D'3',IOLEDGREEN
		BSF		PWM_TBL+D'19',IOLEDGREEN
		BSF		PWM_TBL+D'11',IOLEDGREEN
		BSF		PWM_TBL+D'27',IOLEDGREEN
		BSF		PWM_TBL+D'7',IOLEDGREEN
		BSF		PWM_TBL+D'23',IOLEDGREEN
		BSF		PWM_TBL+D'15',IOLEDGREEN
		XORLW	B'00011111'
		ADDWF	PCL,F
		BCF		PWM_TBL+D'15',IOLEDGREEN
		BCF		PWM_TBL+D'23',IOLEDGREEN
		BCF		PWM_TBL+D'7',IOLEDGREEN
		BCF		PWM_TBL+D'27',IOLEDGREEN
		BCF		PWM_TBL+D'11',IOLEDGREEN
		BCF		PWM_TBL+D'19',IOLEDGREEN
		BCF		PWM_TBL+D'3',IOLEDGREEN
		BCF		PWM_TBL+D'29',IOLEDGREEN
		BCF		PWM_TBL+D'13',IOLEDGREEN
		BCF		PWM_TBL+D'21',IOLEDGREEN
		BCF		PWM_TBL+D'5',IOLEDGREEN
		BCF		PWM_TBL+D'25',IOLEDGREEN
		BCF		PWM_TBL+D'9',IOLEDGREEN
		BCF		PWM_TBL+D'17',IOLEDGREEN
		BCF		PWM_TBL+D'1',IOLEDGREEN
		BCF		PWM_TBL+D'30',IOLEDGREEN
		BCF		PWM_TBL+D'14',IOLEDGREEN
		BCF		PWM_TBL+D'22',IOLEDGREEN
		BCF		PWM_TBL+D'6',IOLEDGREEN
		BCF		PWM_TBL+D'26',IOLEDGREEN
		BCF		PWM_TBL+D'10',IOLEDGREEN
		BCF		PWM_TBL+D'18',IOLEDGREEN
		BCF		PWM_TBL+D'2',IOLEDGREEN
		BCF		PWM_TBL+D'28',IOLEDGREEN
		BCF		PWM_TBL+D'12',IOLEDGREEN
		BCF		PWM_TBL+D'20',IOLEDGREEN
		BCF		PWM_TBL+D'4',IOLEDGREEN
		BCF		PWM_TBL+D'24',IOLEDGREEN
		BCF		PWM_TBL+D'8',IOLEDGREEN
		BCF		PWM_TBL+D'16',IOLEDGREEN
		BCF		PWM_TBL+D'0',IOLEDGREEN
;		BCF		PWM_TBL+D'31',IOLEDGREEN
		RETURN
MAKEPATTRN_BLUE
		ANDLW	B'00011111'
		ADDWF	PCL,F
		BSF		PWM_TBL+D'15',IOLEDBLUE
		BSF		PWM_TBL+D'31',IOLEDBLUE
		BSF		PWM_TBL+D'0',IOLEDBLUE
		BSF		PWM_TBL+D'16',IOLEDBLUE
		BSF		PWM_TBL+D'8',IOLEDBLUE
		BSF		PWM_TBL+D'24',IOLEDBLUE
		BSF		PWM_TBL+D'4',IOLEDBLUE
		BSF		PWM_TBL+D'20',IOLEDBLUE
		BSF		PWM_TBL+D'12',IOLEDBLUE
		BSF		PWM_TBL+D'28',IOLEDBLUE
		BSF		PWM_TBL+D'2',IOLEDBLUE
		BSF		PWM_TBL+D'18',IOLEDBLUE
		BSF		PWM_TBL+D'10',IOLEDBLUE
		BSF		PWM_TBL+D'26',IOLEDBLUE
		BSF		PWM_TBL+D'6',IOLEDBLUE
		BSF		PWM_TBL+D'22',IOLEDBLUE
		BSF		PWM_TBL+D'14',IOLEDBLUE
		BSF		PWM_TBL+D'30',IOLEDBLUE
		BSF		PWM_TBL+D'1',IOLEDBLUE
		BSF		PWM_TBL+D'17',IOLEDBLUE
		BSF		PWM_TBL+D'9',IOLEDBLUE
		BSF		PWM_TBL+D'25',IOLEDBLUE
		BSF		PWM_TBL+D'5',IOLEDBLUE
		BSF		PWM_TBL+D'21',IOLEDBLUE
		BSF		PWM_TBL+D'13',IOLEDBLUE
		BSF		PWM_TBL+D'29',IOLEDBLUE
		BSF		PWM_TBL+D'3',IOLEDBLUE
		BSF		PWM_TBL+D'19',IOLEDBLUE
		BSF		PWM_TBL+D'11',IOLEDBLUE
		BSF		PWM_TBL+D'27',IOLEDBLUE
		BSF		PWM_TBL+D'7',IOLEDBLUE
		BSF		PWM_TBL+D'23',IOLEDBLUE
		XORLW	B'00011111'
		ADDWF	PCL,F
		BCF		PWM_TBL+D'23',IOLEDBLUE
		BCF		PWM_TBL+D'7',IOLEDBLUE
		BCF		PWM_TBL+D'27',IOLEDBLUE
		BCF		PWM_TBL+D'11',IOLEDBLUE
		BCF		PWM_TBL+D'19',IOLEDBLUE
		BCF		PWM_TBL+D'3',IOLEDBLUE
		BCF		PWM_TBL+D'29',IOLEDBLUE
		BCF		PWM_TBL+D'13',IOLEDBLUE
		BCF		PWM_TBL+D'21',IOLEDBLUE
		BCF		PWM_TBL+D'5',IOLEDBLUE
		BCF		PWM_TBL+D'25',IOLEDBLUE
		BCF		PWM_TBL+D'9',IOLEDBLUE
		BCF		PWM_TBL+D'17',IOLEDBLUE
		BCF		PWM_TBL+D'1',IOLEDBLUE
		BCF		PWM_TBL+D'30',IOLEDBLUE
		BCF		PWM_TBL+D'14',IOLEDBLUE
		BCF		PWM_TBL+D'22',IOLEDBLUE
		BCF		PWM_TBL+D'6',IOLEDBLUE
		BCF		PWM_TBL+D'26',IOLEDBLUE
		BCF		PWM_TBL+D'10',IOLEDBLUE
		BCF		PWM_TBL+D'18',IOLEDBLUE
		BCF		PWM_TBL+D'2',IOLEDBLUE
		BCF		PWM_TBL+D'28',IOLEDBLUE
		BCF		PWM_TBL+D'12',IOLEDBLUE
		BCF		PWM_TBL+D'20',IOLEDBLUE
		BCF		PWM_TBL+D'4',IOLEDBLUE
		BCF		PWM_TBL+D'24',IOLEDBLUE
		BCF		PWM_TBL+D'8',IOLEDBLUE
		BCF		PWM_TBL+D'16',IOLEDBLUE
		BCF		PWM_TBL+D'0',IOLEDBLUE
		BCF		PWM_TBL+D'31',IOLEDBLUE
;		BCF		PWM_TBL+D'15',IOLEDBLUE
		RETURN
PGID
		DT		"SolorGuideLamp0.5.4d(c)2006 suzumesyako.jp"

;===============================
;         Init
;===============================
;		ORG	0x0100
START	
;
; === Initialize ===
;
;		*** System clock ***
;
;				  IOC - Interrupt-on-change GPIO register
;				  -110x000  Initial
;				  .XXX....  IRCF: Internal oscillator
;				  .000....     31KHz
;				  .001....     125KHz
;				  .010....     250KHz
;				  .011....     500KHz
;				  .100....     1MHz
;				  .101....     2MHz
;				  .110....     4MHz
;				  .111....     8MHz
;				  ....X...  OSTS:Oscillator start-up time-out status
;				  ....1...     external
;				  ....0...     internal
;				  .....X..  HTS :HFINTOSC status
;				  .....1..     stable
;				  .....0..     not stable
;				  ......X.  LTS :LFINTOSC status
;				  ......1.     stable
;				  ......0.     not stable
;				  .......X  SCS :System clock select
;				  .......1     internal oscillator
;				  .......0     by FOSC
		MOVLW	B'01000001'		; 1MHz/Internal/no output
;		MOVLW	B'00000001'		; 31KHz/Internal/no output
		BANKSEL	OSCCON			; OSC Control
		MOVWF	OSCCON			; OSC Control

;		*** System option ***
;
;				  OPTION_REG - Option register
;				  11111111  Initial
;				  X.......  GPPU: GPIO Pull-up enable
;				  1.......     disabled
;				  0.......     enable
;				  .X......  INTEDG: Interrupt edge select
;				  .1......     riging edge
;				  .0......     failling edge
;				  ..X.....  T0CS: TMR0 clock source select
;				  ..1.....     Transition on GP2/T0CKI pin
;				  ..0.....     Internal instruction cycle
;				  ...X....  T0SE: TMR0 source edge select
;				  ...1....     high-to-low edge
;				  ...0....     low-to-high edge
;				  ....X...  PSA: Prescaler assignment
;				  ....1...     WDT
;				  ....0...     Timer0
;				  .....XXX  PS:  Prescaler assignment
;				  .....000     TMR0 rate 1:2 / WDT rate 1:1
;				  .....001               1:4 /          1:2
;				  .....010               1:8 /          1:4
;				  .....011              1:16 /          1:8
;				  .....100              1:32 /         1:16
;				  .....101              1:64 /         1:32
;				  .....110             1:128 /         1:64
;				  .....111             1:256 /        1:128
		MOVLW	B'11011111'		; Timer0
		BANKSEL	OPTION_REG
		MOVWF	OPTION_REG

;		*** Comparator ***
;
;				  CMCON0 - Comparator control register 0
;				  -0-00000  Initial
;				  .X......  COUT: Comparator output
;				  .1......     Vin+ > Vin-  CINV=0
;				  .0......     Vin+ < Vin-  CINV=0
;				  ...X....  CINV: Comparator output inversion
;				  ...1....     invert
;				  ...0....     normal
;				  ....X...  CIS : Comparator input switch
;				  ....1...     Vin- connects to CIN+
;				  ....0...     Vin- connects to CIN-
;				  .....XXX  CM  : Comparator mode
;				  .....000     reset
;				  .....001     with output
;				  .....010     without output
;				  .....011     without output&internal reference
;				  .....100     w/o output&with internal reference
;				  .....101     multiplexed input with internal reference&output
;				  .....110     multiplexed input with internal reference
;				  .....111     off
		MOVLW	B'00000111'		; Comparator Off
		BANKSEL	CMCON0
		MOVWF	CMCON0

;		*** A/D Converter ***
;
;				  ANSEL - Analog select register
;				  -0001111  Initial
;				  .XXX....  ADCS: A/D converter clock select
;				  .000....     Fosc/2
;				  .001....     Fosc/8
;				  .010....     Fosc/32
;				  .x11....     FRC
;				  .100....     Fosc/4
;				  .101....     Fosc/16
;				  .110....     Fosc/64
;				  ....X...  ANS3: analog input assign AN3/GP4
;				  .....X..  ANS2: analog input assign AN2/GP2
;				  ......X.  ANS1: analog input assign AN1/GP1
;				  .......X  ANS0: analog input assign AN0/GP0
		MOVLW	B'00110011'		; FRC & AN0,AN1
		BANKSEL	ANSEL			; Set GP to analog
		MOVWF	ANSEL			; Set GP to analog

;		*** GPIO port ***
;
;				  GPIO - General purpose I/O register
;                 --xxxxxx  Initial
;				  ..X.....  GP5:
;				  ...X....  GP4:
;				  ....X...  GP3:
;				  .....X..  GP2:
;				  ......X.  GP1:
;				  .......X  GP0:
		MOVLW	LEDALLOFF		; All off
		BANKSEL	GPIO
		MOVWF	GPIO

;		*** Port in/out ***
;
;				  TRISIO - GPIO tri-state register
;                 --111111  Initial
;				  ..X.....  TRISIO5: GP5 1=input 0=output
;				  ...X....  TRISIO4: GP4 1=input 0=output
;				  ....X...  TRISIO3: GP3 1=input only
;				  .....X..  TRISIO2: GP2 1=input 0=output
;				  ......X.  TRISIO1: GP1 1=input 0=output
;				  .......X  TRISIO0: GP0 1=input 0=output
		MOVLW	B'00001011'		; GP1,3:input GP2,4,5:output GP0:Dynamic
		BANKSEL	TRISIO			; Static setup is low power, LED port
		MOVWF	TRISIO			; Static setup is low power, LED port

;
;				  WPU - Weak pull-up register
;                 00110111  Initial
;				  ..X.....  WPU5: GP5 1=enabled 0=disabled
;				  ...X....  WPU4: GP4 1=enabled 0=disabled
;				  .....X..  WPU2: GP2 1=enabled 0=disabled
;				  ......X.  WPU1: GP1 1=enabled 0=disabled
;				  .......X  WPU0: GP0 1=enabled 0=disabled
		MOVLW	B'00000000'		; All off
		BANKSEL	WPU				; Must be OPTION_REG:GPPU
		MOVWF	WPU				; Must be OPTION_REG:GPPU

;		*** Timer 0 ***
;
;		TMR0 30.27Hz=33.0ms (31KHz clock)
;		TMR0 122.07Hz=8.2ms (125KHz clock)
;		TMR0 244.14Hz=4.1ms (250KHz clock)
;		TMR0 488.28Hz=2.0ms (500KHz clock)
;		TMR0 976.56Hz=1.0ms (1MHz clock)


;		*** Timer 1 ***
;
		BANKSEL	TMR1H			; 
		MOVLW	TMR1VH			; 
		MOVWF	TMR1H			; 
		MOVLW	TMR1VL			; 
		MOVWF	TMR1L			; 
;				  T1CON - Timer1 control register
;                 00000000  Initial
;				  X.......  T1GENV: Gate invert
;				  .X......  TMR1GE: Gate enable
;				  ..XX....  T1CKPS: Input clock prescale
;				  ..11....     1:8
;				  ..10....     1:4
;				  ..01....     1:2
;				  ..00....     1:1
;				  ....X...  T1OSCEN:LP Oscillator enable
;				  .....X..  T1SYNC: External clock synchronization control
;				  .....1..     Do not sync  TMR1CS=1 
;				  .....0..     sync         TMR1CS=1
;				  ......X.  TMR1CS: Clock source select
;				  ......1.     External - T1CKI pin
;				  ......0.     Internal - Fosc/4
;				  .......X  TMR1ON: Timer1 on bit
;				  .......1     enable
;				  .......0     stop
		MOVLW	B'00000000'		; off
		BANKSEL	T1CON
		MOVWF	T1CON

;		*** Timer 2 ***
;
		MOVLW	LAMPPERIOD		; PWM period = (Fosc/4 unit)
		BANKSEL	PR2
		MOVWF	PR2

;				  T2CON - Timer2 control register
;                 00000000  Initial
;				  .XXXX...  TOUTPS: Timer2 output postscale select bits
;				  .0000...     1:1
;				  .0001...     1:2
;				     :
;				  .1111...     1:16
;				  .....X..  TMR2ON: Timer2 on bit
;				  ......XX  T2CKPS: Timer2 clock prescale select bits
;				  ......00     Prescaler is 1
;				  ......01     Prescaler is 4
;				  ......1x     Prescaler is 16
		MOVLW	B'00000000'		; off
		BANKSEL	T2CON
		MOVWF	T2CON

;		*** CCP ***
;
		MOVLW	LAMPDARK		; PWM duty cycle rate, initial
		BANKSEL	CCPR1L
		MOVWF	CCPR1L
;				  CCP1CON - CCP control register 1
;                 00000000  Initial
;				  ..XX....  DC1B: PWM Least significant bits
;				  ....XXXX  CCP1M: CCP1 Mode select bits
;				  ....0000     disable(reset)
;				  ....0100     Capture mode, every failing edge
;				  ....0101     Capture mode, every riging edge
;				  ....0110     Capture mode, every 4th edge
;				  ....0111     Capture mode, every 16th edge
;				  ....1000     Compare mode, set output match
;				  ....1001     Compare mode, clear output match
;				  ....1010     Compare mode, generate software intrrupt on match
;				  ....1011     Compare mode, trigger special event
;				  ....110x     PWM mode active high
;				  ....111x     PWM mode active low
		MOVLW	B'00000000'		; off
		BANKSEL	CCP1CON
		MOVWF	CCP1CON

;		*** Interrupt ***
;
;				  IOC - Interrupt-on-change GPIO register
;                 --000000  Initial
;							0=disable 1=enable
;				  ..X.....  IOC5:GP5
;				  ...X....  IOC4:GP4
;				  ....X...  IOC3:GP3
;				  .....X..  IOC2:GP2
;				  ......X.  IOC1:GP1
;				  .......X  IOC0:GP0
		MOVLW	B'00001000' 
		BANKSEL	IOC
		MOVWF	IOC

;				  PIE1 - Peripheral interrupt enable register1
;                 -0000000  Initial
;							0=disable 1=enable
;				  .X......  EEIE  : EEPROM Write complete
;				  ..X.....  ADIE  : A/D Converter interrupt
;				  ...X....  CCP1IE: CCP1 interrupt
;				  ....X...  CMIE  : Comparator interrupt
;				  .....X..  OSFIE : Oscillator fail
;				  ......X.  TMR2IE: Timer2 overflow
;				  .......X  TMR1IE: Timer1 overflow
		MOVLW	B'00000000'		; Off
		BANKSEL	PIE1
		MOVWF	PIE1

;				  INTCON - Interrupt control register
;                 00000000  Initial
;							0=disable 1=enable
;				  X.......  GIE : Global interrupt
;				  .X......  PEIE: Peripheral intterupt -> PIE1
;				  ..X.....  T0IE: TMR0 Overflow interrupt
;				  ...X....  INTE: P2/INT external interrupt
;				  ....X...  GPIE: GPIO Change interrupt -> IOC
;							0=not(Appl reset) 1=occurred
;				  .....X..  T0IF: TMR0 Overflow interrupt flag
;				  ......X.  INTF: GP2/INT external interrupt flag
;				  .......X  GPIF: GPIO Change interrupt flag
		MOVLW	B'10001000'		; IOC+TMR0(dynamic)
		BANKSEL	INTCON
		MOVWF	INTCON

;		*** Wach dog timer ***
;
;				  WDTCON - Watchdog timer control register
;                 ---01000  Initial
;				  ...XXXX.  WDTPS: Watchdog timer period select
;				  ...0000.     1:32
;				  ...0001.     1:64
;				  ...0010.     1:128
;				  ...0011.     1:256
;				  ...0100.     1:512
;				  ...0101.     1:1024
;				  ...0110.     1:2048
;				  ...0111.     1:4096
;				  ...1000.     1:8192
;				  ...1001.     1:16384
;				  ...1010.     1:32768
;				  ...1011.     1:65536
;				  .......X  SWDTEN:Software enable
;				  .......1     WDT is turned on
;				  .......0     WDT is turned off
		MOVLW	B'00000111'		; about 1.0sec and WDT start
		BANKSEL	WDTCON
		MOVWF	WDTCON

;		*** Ready ***
;
		BANKSEL	EEADR			; EEPROM Write location
		CLRF	EEADR			; EEPROM Write location
		BANKSEL	USERWORK		; Bank 0 swtch
		CLRF	LAMPTIMEON1H
		CLRF	LAMPTIMEON1L
		CLRF	LAMPTIMEON2H
		CLRF	LAMPTIMEON2L
		CLRF	PFLAG
		CLRF	LAMPFLAG
		CLRF	LAMPWFLAG
		CLRF	PWM_DUTY
		CLRF	SCAN_STATE

		MOVLW	PWM_TBL
		MOVWF	FSR
		MOVLW	D'32'
		MOVWF	PWM_CTR
		MOVLW	LEDALLOFF
INIT10
		MOVWF	INDF
		INCF	FSR
		DECFSZ	PWM_CTR
		GOTO	INIT10

;===============================
;     Main
;===============================
MAINLOOP						; 1loop = 1sec
		CLRWDT

; State counter
		MOVF	SCAN_STATE,W
		INCF	SCAN_STATE

; State 0
		ANDLW	H'F'
		BTFSS	STATUS,Z
		GOTO	MAIN10

		CALL	AN1IN			; AN1:Battery voltage read
		MOVF	VBATT			; Save state0 timming
		BANKSEL	TRISIO			; Bank 1 swtch
		BSF		TRISIO,IOSCGATE	; GP0/AN0: mode change to input
		BANKSEL	USERWORK		; Bank 0 swtch
		GOTO	MAIN90

; State 1
MAIN10
		XORLW	D'1'
		BTFSS	STATUS,Z
		GOTO	MAIN20

		CALL	AN1IN			; AN1:Battery voltage read
		CALL	AN0IN			; AN0:Soler voltage read
		BANKSEL	TRISIO			; Bank 1 swtch
		BCF		TRISIO,IOSCGATE	; GP0/AN0: i/o mode recall
		BANKSEL	USERWORK		; Bank 0 swtch

		CALL 	BATTCTL			; Battery charge on/off
		CALL	BRIGHTCHK		; Day Night check
;		CALL	PIRCTL			; PIR power on/off
		CALL	LAMPCTL			; Lamp on/off
		GOTO	MAIN90

; State 2-15
MAIN20

; All State
MAIN90
		CALL	LIGHTCTL		; Lamp one shot timer

;
;		Waiting or LED refresh
;
MAIN100
		BTFSC	PFLAG,1			; LED on ?
		GOTO	MAIN120
		BTFSC	LAMPFLAG,5		; LED on request ?
		GOTO	MAIN120

;
;		Power saveing, ABOUT 1 sec cycle
;
MAIN110
		SLEEP					; Waiting WDT wakeup
		GOTO	MAINLOOP
;
;		Interval wait and LED refresh, ABOUT 1sec
;
MAIN120
		MOVLW	D'20'			; 20count * 50msec = 1.0sec
;		MOVLW	D'10'			; 10count * 100msec = 1.0sec
;		MOVLW	D'8'			; 8 count * 125msec = 1.0sec
;		MOVLW	D'7'			; 7 count * 143msec = 1.0sec
;		MOVLW	D'5'			; 5 count * 200msec = 1.0sec
;		MOVLW	D'2'			; 2 count * 500msec = 1.0sec
		MOVWF	MAIN_CTR
MAIN125
		CALL	LAMP_REFRESH	; Main LED PWM drive
		MOVLW	D'50'			; msec
;		MOVLW	D'100'			; msec
;		MOVLW	D'125'			; msec
;		MOVLW	D'143'			; msec
;		MOVLW	D'200'			; msec
;		MOVLW	D'250'			; msec
;		CALL	WAIT
;		MOVLW	D'250'			; msec
		CALL	WAIT
		DECFSZ	MAIN_CTR,F
		GOTO	MAIN125

		GOTO	MAINLOOP

;===============================
;     Analog port sampling
;===============================
AN0IN
;		Sampling
		MOVLW	B'00000001'		; Left, Vdd Vref, AN0
		CALL	ANYIN
		MOVWF	AN0VAL
;		CALL	EEPROM			; ***DEBUG***
;		MOVF	AN0VAL,W		; ***DEBUG***
;		<Vsc> = (Vbatt-GP0)*(R5+R6)/R5 = (AN1-AN0)*(1M+2M)/1M = (AN1-AN0)*3
		SUBWF	AN1VAL,W		; AN1VAL - W -> W
		MOVWF	AN0VAL			; W -> AN0VAL
		ADDWF	AN0VAL,F		; AN0VAL + W -> AN0VAL
		ADDWF	AN0VAL,F		; AN0VAL + W -> AN0VAL
		MOVLW	D'11'			; Adjust -0.15V
		SUBWF	AN0VAL,F
		MOVF	AN0VAL,W
;		CALL	EEPROM			; ***DEBUG***
;		MOVF	AN0VAL,W		; ***DEBUG***
		RETURN

AN1IN
;		Sampling
		MOVLW	B'00000101'		; Left, Vdd Vref, AN1
		CALL	ANYIN
		MOVWF	AN1VAL
;		CALL	EEPROM			; ***DEBUG***
;		MOVF	AN1VAL,W		; ***DEBUG***
		RETURN

; Sampling miss adjust
ANYIN
		MOVWF	ANYCMD			; Command save

		CALL	ANXIN			; First sampling
		MOVWF	ANYVAL1			; Value save

		MOVF	ANYCMD,W		; Command recall
		CALL	ANXIN			; 2nd sampling
		MOVWF	ANYVAL2			; Value save

		MOVF	ANYVAL1,W		; 
		MOVWF	ANYVALS			; ANYVALS := ANYVAL1
		MOVWF	ANYVALH			; ANYVALH := ANYVAL1
		MOVWF	ANYVALL			; ANYVALL := ANYVAL1

		BCF		STATUS,C		; ANYVALS := ANYVAL1 / 8
		RRF		ANYVALS,F
		BCF		STATUS,C
		RRF		ANYVALS,F
		BCF		STATUS,C
		RRF		ANYVALS,F

		MOVF	ANYVALS,W
		ADDWF	ANYVALH,F		; ANYVALH := ANYVAL1 + ANYVALS
		SUBWF	ANYVALL,F		; ANYVALL := ANYVAL1 - ANYVALS

		MOVF	ANYVALH,W		; W := ANYVAL2 - ANYVALH
		SUBWF	ANYVAL2,W
		BTFSC	STATUS,C
		GOTO	ANYIN20

		MOVF	ANYVALL,W		; W := ANYVAL2 - ANYVALL
		SUBWF	ANYVAL2,W
		BTFSS	STATUS,C
		GOTO	ANYIN20
ANYIN10							; VAL1 or VAL2 correct
		MOVF	ANYVAL1,W		; Set value VAL1
		GOTO	ANYIN90
ANYIN20
		MOVF	ANYCMD,W		; Command recall
		CALL	ANXIN			; 3rd sampling
		MOVWF	ANYVAL3			; Value save

		MOVF	ANYVALH,W		; W := ANYVAL3 - ANYVALH
		SUBWF	ANYVAL3,W
		BTFSC	STATUS,C
		GOTO	ANYIN40

		MOVF	ANYVALL,W		; W := ANYVAL3 - ANYVALL
		SUBWF	ANYVAL3,W
		BTFSS	STATUS,C
		GOTO	ANYIN40
ANYIN30							; VAL1 or VAL3 correct
		MOVF	ANYVAL3,W		; Set value VAL3
		GOTO	ANYIN90

ANYIN40							; VAL1 is bad
		MOVF	ANYVAL2,W		; Set value VAL2
		GOTO	ANYIN90
ANYIN90							; Exit
		RETURN

; Simple sampling
ANXIN
		MOVWF	ADCON0			; Write sampling start command
		NOP
		NOP
		BSF		ADCON0,GO		; Start conversion
		BTFSC	ADCON0,GO		; Is conversion done?
		GOTO	$-1				; No, test again
		MOVF	ADRESH,W		; Read upper 8 bits, W is value
		MOVWF	ANXVAL			; Save
		MOVLW	B'00000000'		; ADC OFF
		MOVWF	ADCON0			;
		MOVF	ANXVAL,W		; Recall
		RETURN

;===============================
;     Battry charge control
;===============================
BATTCTL

;		Low check
		MOVF	AN1VAL,W		; AN1: Battery voltage
		SUBLW	BATTLOWD
		BTFSS	STATUS,C		; Low limit under ?
		GOTO	BATTCTL10		; 
		BSF		PFLAG,0			; Power down flag
		GOTO	BATTCTL_EXIT
BATTCTL10
		BTFSS	PFLAG,0			; Power down now ?
		GOTO	BATTCTL20		; Low check end
		MOVF	AN1VAL,W		; AN1: Battery voltage
		SUBLW	BATTLOWU
		BTFSS	STATUS,C		; Upper ?
		BCF		PFLAG,0			; Power down reset

;		High check
BATTCTL20
		MOVF	AN1VAL,W		; AN1: Battery voltage
		SUBLW	BATTHIGHU
		BTFSC	STATUS,C		; High limit over ?
		GOTO	BATTCTL30
								; Over charge
		BSF		PFLAG,4			; Over load flag on

;		Charge off / cut

;		MOVLW	LAMPHIDUTY3		; AND 100% duty by main LED
;		MOVWF	CCPR1L			; Set PWM port
;		BANKSEL	TRISIO			; Bank 1 swtch
;		BCF		TRISIO,IOLAMP	; Port output (PWM on)
;		BANKSEL	USERWORK		; Bank 0 swtch

;		MOVLW	LEDALLON
;		MOVWF	GPIO

;		BANKSEL	TRISIO			; Bank 1 swtch
;		BCF		TRISIO,IOLEDRED	; Port output mode
;		BCF		TRISIO,IOLEDGREEN
;		BCF		TRISIO,IOLEDBLUE
;		BANKSEL	USERWORK		; Bank 0 swtch

		GOTO	BATTCTL_EXIT

BATTCTL30						; High limit under
		BTFSS	PFLAG,4			; Over load now ?
		GOTO	BATTCTL_EXIT
		MOVF	AN1VAL,W		; AN1: Battery voltage
		SUBLW	BATTHIGHD
		BTFSC	STATUS,C		; Under ?
		GOTO	BATTCTL_EXIT
								; Over charge recoverd
		BCF		PFLAG,4			; Over load flag off

;		ReCharge

;		BANKSEL	TRISIO			; Bank 1 swtch
;		BSF		TRISIO,IOLAMP	; Port tri-state (PWM off)
;		BANKSEL	USERWORK		; Bank 0 swtch

;		MOVLW	LEDALLOFF
;		MOVWF	GPIO

;		BANKSEL	TRISIO			; Bank 1 swtch
;		BSF		TRISIO,IOLEDRED	; Port input mode
;		BSF		TRISIO,IOLEDGREEN
;		BSF		TRISIO,IOLEDBLUE
;		BANKSEL	USERWORK		; Bank 0 swtch

BATTCTL_EXIT
;
;		FET Gate control
;
		BTFSC	PFLAG,4			; Over load flag
		GOTO	BATTCTL_GATEOFF

;		Absolute method
;		Vsc > Vbatt -> on
		MOVF	AN1VAL,W
		SUBWF	AN0VAL,W		; AN0VAL(Vsc) - AN1VAL(Vbatt) -> W
		BTFSS	STATUS,C		;
		GOTO	BATTCTL_GATEOFF
;		GOTO	BATTCTL_GATEON
;
;;		Differential method
;;		Vbatt(load) < Vbatt(free)  -> gate reverse on/off
;		MOVF	VBATT,W
;		SUBWF	AN1VAL,W		; AN1VAL(Vbatt load) - VBATT(free) -> W
;		BTFSS	STATUS,C		; borrow skip
;		GOTO	BATTCTL_99		; 
;		BTFSC	GPIO,IOSCGATE	; 0->1 1->0
;		GOTO	BATTCTL_GATEOFF
;		GOTO	BATTCTL_GATEON

BATTCTL_GATEON
		BSF		GPIO,IOSCGATE
		GOTO	BATTCTL_99
BATTCTL_GATEOFF
		BCF		GPIO,IOSCGATE
;		GOTO	BATTCTL_99
BATTCTL_99
		RETURN


;            ..
;          **  ..  <--HIGHU   
;        **      ..                   **
;      **          **  <---HIGHD    **
;    **              **           **
;  **                  **        **  <--- LOWU
;                        **    ..
;                          ....  <--- LOWD
;
;  SSSSSSSSSS......SSSSSSSSSSSSSSSSSSSSS <= Enable solar cell
;  PPPPPPPPPPPPPPPPPPPPPPPP......PPPPPPP <= Enable All function

;===============================
;     World bright check, and LED on/off
;===============================
BRIGHTCHK
BRIGHTCHK_BATCHK				; Battery low level check
		BTFSC	PFLAG,0			; Power fail ?
		GOTO	BRIGHTCHK90		; Low batt, skip
		MOVF	AN0VAL,W		; Soler power voltage
		SUBLW	BRIGHTNIGHT		; Lower voltage
		BTFSS	STATUS,C		; Night ?
		GOTO	BRIGHTCHK10
BRIGHTCHK_NIGHT
		BSF		PFLAG,3			; Night mode
		GOTO	BRIGHTCHK90
BRIGHTCHK10
		BTFSS	PFLAG,3			; Night ?
		GOTO	BRIGHTCHK90		; Low check end
		MOVF	AN0VAL,W		; AN0: Soler power voltage
		SUBLW	BRIGHTDAY		; Higher voltage
		BTFSC	STATUS,C		; Upper ?
		GOTO	BRIGHTCHK90
BRIGHTCHK_DAY
		BCF		PFLAG,3			; Day mode
BRIGHTCHK90
		RETURN

;===============================
;     PIR power control
;===============================
;PIRCTL
;PIRCTL_BATCHK					; Battery low level check
;		BTFSC	PFLAG,0			; Power fail ?
;		GOTO	PIRCTL_EXIT		; Low batt, skip
;		BTFSS	PFLAG,3			; Night ?
;		GOTO	PIRCTL_OFF
;PIRCTL_ON
;		BTFSS	GPIO,IOPIRPWR	; PIR ON ?
;		BSF		GPIO,IOPIRPWR	; PIR power ON
;		GOTO	PIRCTL_EXIT
;PIRCTL_OFF
;		BTFSC	GPIO,IOPIRPWR	; PIR OFF ?
;		BCF		GPIO,IOPIRPWR	; PIR power OFF
;PIRCTL_EXIT
;		RETURN

;===============================
;     Lamp on/off control
;===============================
LAMPCTL
LAMPCTL_BATCHK					; Battery low level check
		BTFSC	PFLAG,0			; Power fail ?
		GOTO	LAMPCTL_EXIT	; Low batt, skip
		BTFSS	PFLAG,3			; Night ?
		GOTO	LAMPCTL_OFF
LAMPCTL_ON
		BTFSC	LAMPFLAG,5		; Already ON ?
		GOTO	LAMPCTL_EXIT
		BSF		LAMPFLAG,5		; Flag on
		CALL	LAMP_TIMER_START; Normal Lamp on, start

		MOVLW	D'13'			; Lamp color set   2007.1.3
		MOVWF	PWM_COLOR
LAMPCTL_ON10
		MOVF	PWM_COLOR,W
		CALL	VOLCOLOR
		SUBWF	AN1VAL,W
		BTFSC	STATUS,C
		GOTO	LAMPCTL_ON20
		DECFSZ	PWM_COLOR
		GOTO	LAMPCTL_ON10
LAMPCTL_ON20
;;
;		MOVLW	BATTLOWD		; *DEBUG*
;		SUBWF	AN1VAL,W		; *DEBUG* Today color
;		MOVWF	PWM_COLOR		; *DEBUG*
;		RRF		PWM_COLOR,F		; *DEBUG*
;;		RRF		PWM_COLOR,F		; *DEBUG* GoldCap
;;		RRF		PWM_COLOR,F		; *DEBUG* GoldCap
;;		RRF		PWM_COLOR,F		; *DEBUG* GoldCap
;;		RRF		PWM_COLOR,F		; *DEBUG* GoldCap
;;
		GOTO	LAMPCTL_EXIT
LAMPCTL_OFF
		BTFSS	LAMPFLAG,5		; Already OFF ?
		GOTO	LAMPCTL_EXIT
		BCF		LAMPFLAG,5		; Flag off
		CALL	LAMP_TIMER_STOP	; Normal Lamp off, stop
LAMPCTL_EXIT
		RETURN

;===============================
;     Lamp start/stop ovent
;===============================
LAMP_TIMER_START				; Sunset event
		MOVLW	LAMPONTIME1H	; Timer 1 set
		MOVWF	LAMPTIMEON1H
		MOVLW	LAMPONTIME1L
		MOVWF	LAMPTIMEON1L
		MOVF	AN0VAL,W		; Twilight solar cell vol
		CALL	EEPROM			; Logging
		MOVF	AN1VAL,W		; Today battery voltage
		CALL	EEPROM			; Logging
;		MOVLW	h'88'			; ***DEBUG***
;		CALL	EEPROM			; ***DEBUG***
		RETURN

LAMP_TIMER_START2				; PIR sense event
		MOVLW	LAMPONTIME2H	; Timer 2 set
		MOVWF	LAMPTIMEON2H
		MOVLW	LAMPONTIME2L
		MOVWF	LAMPTIMEON2L
		RETURN

LAMP_TIMER_STOP
		CLRF	LAMPTIMEON1H	; Timer 1 reset
		CLRF	LAMPTIMEON1L
		CLRF	LAMPTIMEON2H	; Timer 2 reset
		CLRF	LAMPTIMEON2L
		RETURN

;===============================
;     Lamp auto off & slow flush control
;===============================
LIGHTCTL
;		soft timer count down and flag check
		MOVF	LAMPWFLAG,W
		MOVWF	LAMPWFLAGB		; save old flags
		CLRF	LAMPWFLAG

		MOVF	LAMPTIMEON1H,W	; timer1 <> 0 ?
		IORWF	LAMPTIMEON1L,W
		BTFSC	STATUS,Z
		GOTO	LIGHTCTL10
		MOVLW	D'1'			; timer1--
		SUBWF	LAMPTIMEON1L,F
		BTFSS	STATUS,C
		SUBWF	LAMPTIMEON1H,F
		BSF		LAMPWFLAG,1		; non zero flag
LIGHTCTL10
		MOVF	LAMPTIMEON2H,W	; timer2 <> 0 ?
		IORWF	LAMPTIMEON2L,W
		BTFSC	STATUS,Z
		GOTO	LIGHTCTL20
		MOVLW	D'1'			; timer2--
		SUBWF	LAMPTIMEON2L,F
		BTFSS	STATUS,C
		SUBWF	LAMPTIMEON2H,F
		BSF		LAMPWFLAG,2		; non zero flag
LIGHTCTL20
		MOVF	LAMPTIMEFL,F	; slow flush timer <> 0 ?
		BTFSC	STATUS,Z
		GOTO	LIGHTCTL30
		DECF	LAMPTIMEFL,F	; slow flush timer --
		BSF		LAMPWFLAG,3		; non zero flag
LIGHTCTL30
; Timer 2 (PIR sense) control
LIGHTCTL_TMR2
		BTFSS	LAMPWFLAG,2
		GOTO	LIGHTCTL_TMR2_TIMEOUT
LIGHTCTL_TMR2_INCOUNT			; now PIR mode
		BTFSC	LAMPWFLAGB,2	; status change ?
		GOTO	LIGHTCTL_TMR2_END	; all ready on, exit
;		start PIR mode flushing
		MOVLW	LAMPHIDUTY2		; Lamp on request
		MOVWF	LAMPHIDUTY
		MOVLW	LAMPLODUTY2
		MOVWF	LAMPLODUTY

		MOVLW	LAMPFOTIME2		; Slow flush rate
		MOVWF	LAMPFOTIME
		MOVLW	LAMPFFTIME2
		MOVWF	LAMPFFTIME

		BCF		LAMPWFLAG,3		; reset
		GOTO	LIGHTCTL_TMR2_END
LIGHTCTL_TMR2_TIMEOUT
		BTFSS	LAMPWFLAGB,2	; status change ?
		GOTO	LIGHTCTL_TMR1	; all ready off, *** continue ***
;		stop PIR mode flushing, and resume other mode
		BTFSC	LAMPWFLAG,1		; timer1 deactive ?
		GOTO	LIGHTCTL_TMR2_RESUME
;		stop flushing
		MOVLW	LAMPDARK		; Lamp cut off request
		MOVWF	LAMPDUTYSET
		GOTO	LIGHTCTL90
LIGHTCTL_TMR2_RESUME
;		resume night mode flushing
		MOVLW	LAMPHIDUTY1		; Lamp on request
		MOVWF	LAMPHIDUTY
		MOVLW	LAMPLODUTY1
		MOVWF	LAMPLODUTY

		MOVLW	LAMPFOTIME1		; Slow flush rate
		MOVWF	LAMPFOTIME
		MOVLW	LAMPFFTIME1
		MOVWF	LAMPFFTIME

		BCF		LAMPWFLAG,3		; reset
		GOTO	LIGHTCTL_TMR2_END

; Timer 1 (night) control
LIGHTCTL_TMR1
		BTFSS	LAMPWFLAG,1
		GOTO	LIGHTCTL_TMR1_TIMEOUT
LIGHTCTL_TMR1_INCOUNT			; now night mode
		BTFSC	LAMPWFLAGB,1	; status change ?
		GOTO	LIGHTCTL_TMR1_END	; all ready on, exit
;		start night mode flushing
		MOVLW	LAMPHIDUTY1		; Lamp on request
		MOVWF	LAMPHIDUTY
		MOVLW	LAMPLODUTY1
		MOVWF	LAMPLODUTY

		MOVLW	LAMPFOTIME1		; Slow flush rate
		MOVWF	LAMPFOTIME
		MOVLW	LAMPFFTIME1
		MOVWF	LAMPFFTIME

		BCF		LAMPWFLAG,3		; reset
		GOTO	LIGHTCTL_TMR1_END
LIGHTCTL_TMR1_TIMEOUT
		BTFSS	LAMPWFLAGB,1	; status change ?
		GOTO	LIGHTCTL90		; all ready off, continue
;		stop flushing
		MOVLW	LAMPDARK		; Lamp cut off request
		MOVWF	LAMPDUTYSET
		GOTO	LIGHTCTL90
LIGHTCTL_TMR1_END

LIGHTCTL_TMR2_END

; slow flash control
LIGHTCTL_SLOWFLUSH
		BTFSC	LAMPWFLAG,3
		GOTO	LIGHTCTL_ON10	; is not zero

		BTFSS	LAMPFLAG,3		; Direction
		GOTO	LIGHTCTL_ON05
								; up -> down
		MOVF	LAMPLODUTY,W	; Lamp on request
		MOVWF	LAMPDUTYSET
		MOVF	LAMPFFTIME,W	; Slow flush timer set
		MOVWF	LAMPTIMEFL
		BCF		LAMPFLAG,3		; Direction is down
		GOTO	LIGHTCTL_ON10
LIGHTCTL_ON05					; up <- down
		MOVF	LAMPHIDUTY,W	; Lamp on request
		MOVWF	LAMPDUTYSET
		MOVF	LAMPFOTIME,W	; Slow flush timer set
		MOVWF	LAMPTIMEFL
		BSF		LAMPFLAG,3		; Direction is up
;		DECF	PWM_COLOR,F		; *DEBUG* 2007.1.3
;		BTFSS	STATUS,Z		; *DEBUG* 2007.1.3
;		GOTO	LIGHTCTL_ON10	; *DEBUG* 2007.1.3
;		MOVLW	D'12'			; *DEBUG* 2007.1.3
;		MOVWF	PWM_COLOR		; *DEBUG* 2007.1.3
LIGHTCTL_ON10

LIGHTCTL90
		RETURN

;
; Phygical control for slow flush light
;
LAMP_REFRESH					; Lamp duty update
		MOVF	LAMPDUTYSET,W
;		SUBWF	CCPR1L,W		; NOW PWM duty
		SUBWF	PWM_DUTY,W		; NOW PWM duty
		BTFSC	STATUS,Z		; NOW == SET
		GOTO	LAMP_REFRESH90
		BTFSS	STATUS,C		; NOW < SET
;		INCF	CCPR1L,F		; NOW++
		INCF	PWM_DUTY,F		; NOW++
		BTFSC	STATUS,C		; NOW > SET
;		DECF	CCPR1L,F		; NOW--
		DECF	PWM_DUTY,F		; NOW--
		MOVF	LAMPDUTYSET,W
; 		LED port enable and disable
;		MOVF	CCPR1L,W		; Now duty value
		MOVF	PWM_DUTY,W		; Now duty value
		SUBLW	LAMPDARK		; OFF level
		BTFSS	STATUS,Z		; NOW == DARK ?
		GOTO	LAMP_REFRESH_ON
LAMP_REFRESH_OFF
		BTFSS	PFLAG,1			; now lamp on ?
		GOTO	LAMP_REFRESH99	; already off, exit
		BCF		PFLAG,1			; Flag set OFF
		BCF		INTCON,T0IE		; PWM OFF
;		Port output mode -> tri-state
;		BANKSEL	TRISIO			; Bank 1 swtch
;		BSF		TRISIO,IOLEDRED	; Port input mode
;		BSF		TRISIO,IOLEDGREEN
;		BSF		TRISIO,IOLEDBLUE
;		BANKSEL	USERWORK		; Bank 0 swtch

		MOVLW	LEDALLOFF		; PWM table clear
		MOVWF	GPIO

		GOTO	LAMP_REFRESH99

LAMP_REFRESH_ON					; Port enable
		BTFSC	PFLAG,1			; now lamp off ?
		GOTO	LAMP_REFRESH90	; already on, continue
		BSF		PFLAG,1			; Flag set ON
		BSF		INTCON,T0IE		; PWM ON
;		Port tri-state mode -> output
;		BANKSEL	TRISIO			; Bank 1 swtch
;		BCF		TRISIO,IOLEDRED	; Port output mode
;		BCF		TRISIO,IOLEDGREEN
;		BCF		TRISIO,IOLEDBLUE
;		BANKSEL	USERWORK		; Bank 0 swtch

LAMP_REFRESH90
;
;		PWM pattrn set
;
		MOVF	PWM_COLOR,W
		CALL	MAKECOLOR_BYTE
		MOVWF	PWM_COLOR_BYTE
		RRF		PWM_DUTY,W
		MOVWF	PWM_WORK

		MOVF	PWM_COLOR_BYTE,W
		ANDLW	B'00110000'			; ..RR....
		BTFSC	STATUS,Z
		GOTO	LAMP_REFRESH91
		ADDLW	B'00010000'
		MOVWF	PWM_WORK2
		BTFSC	PWM_WORK2,5			; quater red
		RRF		PWM_WORK,W			; =duty/4
		BTFSC	PWM_WORK2,4			; half red
		MOVF	PWM_WORK,W			; =duty/2
		BTFSC	PWM_WORK2,6			; full red
		MOVF	PWM_DUTY,W			; =duty
LAMP_REFRESH91
		CALL	MAKEPATTRN_RED		; PWM map

		MOVF	PWM_COLOR_BYTE,W
		ANDLW	B'00001100'			; ....GG..
		BTFSC	STATUS,Z
		GOTO	LAMP_REFRESH92
		ADDLW	B'00000100'
		MOVWF	PWM_WORK2
		BTFSC	PWM_WORK2,3			; quater green
		RRF		PWM_WORK,W			; =duty/4
		BTFSC	PWM_WORK2,2			; half green
		MOVF	PWM_WORK,W			; =duty/2
		BTFSC	PWM_WORK2,4			; full green
		MOVF	PWM_DUTY,W			; =duty
LAMP_REFRESH92
		CALL	MAKEPATTRN_GREEN	; PWM map

		MOVF	PWM_COLOR_BYTE,W
		ANDLW	B'00000011'			; ......BB
		BTFSC	STATUS,Z
		GOTO	LAMP_REFRESH93
		ADDLW	B'00000001'
		MOVWF	PWM_WORK2
		BTFSC	PWM_WORK2,1			; quater blue
		RRF		PWM_WORK,W			; =duty/4
		BTFSC	PWM_WORK2,0			; half blue
		MOVF	PWM_WORK,W			; =duty/2
		BTFSC	PWM_WORK2,2			; full blue
		MOVF	PWM_DUTY,W			; =duty
LAMP_REFRESH93
		CALL	MAKEPATTRN_BLUE		; PWM map

;		BCF		STATUS,C			; RRF be effective
;		MOVLW	D'0'
;		BTFSC	PWM_COLOR_BYTE,4	; half red
;		RRF		PWM_DUTY,W			; =duty/2
;		BTFSC	PWM_COLOR_BYTE,5	; full red
;		MOVF	PWM_DUTY,W			; =duty
;		CALL	MAKEPATTRN_RED		; PWM map

;		BCF		STATUS,C			; RRF be effective
;		MOVLW	D'0'
;		BTFSC	PWM_COLOR_BYTE,2	; half green
;		RRF		PWM_DUTY,W			; =duty/2
;		BTFSC	PWM_COLOR_BYTE,3	; full green
;		MOVF	PWM_DUTY,W			; =duty
;		CALL	MAKEPATTRN_GREEN	; PWM map

;		BCF		STATUS,C			; RRF be effective
;		MOVLW	D'0'
;		BTFSC	PWM_COLOR_BYTE,0	; half blue
;		RRF		PWM_DUTY,W			; =duty/2
;		BTFSC	PWM_COLOR_BYTE,1	; full blue
;		MOVF	PWM_DUTY,W			; =duty
;		CALL	MAKEPATTRN_BLUE		; PWM map

LAMP_REFRESH99
		RETURN


;===============================
;   Intrrupt service routine
;===============================
ISR
		MOVWF	ISR_W			; House keep
		SWAPF	STATUS,W
		CLRF	STATUS
		MOVWF	ISR_S
ISR10
		BTFSC	INTCON,GPIF
		GOTO	ISR_GPIF
		BTFSC	INTCON,INTF
		GOTO	ISR_INTF
		BTFSC	INTCON,T0IF
		GOTO	ISR_T0IF
		BTFSC	PIR1,TMR1IF
		GOTO	ISR_TMR1IF
		BTFSC	PIR1,TMR2IF
		GOTO	ISR_TMR2IF
		BTFSC	PIR1,OSFIF
		GOTO	ISR_OSFIF
		BTFSC	PIR1,CMIF
		GOTO	ISR_CMIF
		BTFSC	PIR1,CCP1IF
		GOTO	ISR_CCP1IF
		BTFSC	PIR1,ADIF
		GOTO	ISR_ADIF
		BTFSC	PIR1,EEIF
		GOTO	ISR_EEIF
ISR20							; Other interrupt
		GOTO	ISR90


ISR_GPIF
		BTFSC	GPIO,IOPIRSIG	; GP3(PIR)Off -> skip
		CALL	LAMP_TIMER_START2

		BCF		INTCON,GPIF
		GOTO	ISR90
ISR_INTF
		BCF		INTCON,INTF
		GOTO	ISR90
ISR_T0IF						; Timer0
		MOVF	PWM_CTR,W		; PWM drive refresh
;		ANDLW	H'07'			; 8 level
;		ANDLW	H'0f'			; 16 level
		ANDLW	H'1f'			; 32 level
		ADDLW	PWM_TBL
		MOVWF	FSR
		MOVF	INDF,W
		MOVWF	GPIO
		INCF	PWM_CTR

		MOVLW	H'80'			; 128 Clock
		MOVWF	TMR0

		BCF		INTCON,T0IF
		GOTO	ISR90
ISR_TMR1IF
		BCF		PIR1,TMR1IF
		GOTO	ISR90
ISR_TMR2IF
		BCF		PIR1,TMR2IF
		GOTO	ISR90
ISR_OSFIF
		BCF		PIR1,OSFIF
		GOTO	ISR90
ISR_CMIF
		BCF		PIR1,CMIF
		GOTO	ISR90
ISR_CCP1IF
		BCF		PIR1,CCP1IF
		GOTO	ISR90
ISR_ADIF
		BCF		PIR1,ADIF
		GOTO	ISR90
ISR_EEIF
		BCF		PIR1,EEIF
		GOTO	ISR90

ISR90
		SWAPF	ISR_S,W			; House keep
		MOVWF	STATUS
		SWAPF	ISR_W,F
		SWAPF	ISR_W,W
		RETFIE

;===============================
;        WAIT (1 ms unit)
;===============================
WAIT
		MOVWF	WAITCTR
WAIT10
;		MOVLW	D'17'			; by 32KHz clock 10ms
;		MOVLW	D'68'			; by 125KHz clock 10ms (not adjust)
;		MOVLW	D'136'			; by 250KHz clock 10ms (not adjust)
;		MOVLW	D'11'			; by 250KHz clock 1ms
;		MOVLW	D'23'			; by 500KHz clock 1ms (not adjust)
		MOVLW	D'46'			; by 1MHz clock 1ms (not adjust)
		MOVWF	WAITCTR2
WAIT20
		CLRWDT					; WDT cancel
		DECFSZ	WAITCTR2,F
		GOTO	WAIT20

		DECFSZ	WAITCTR,F
		GOTO	WAIT10
		RETURN

;===============================
;        EEPROM REGIST
;===============================
EEPROM
		BANKSEL	EECON1

		BTFSC	EECON1,WR			; Freezing EEADR&EEDATA
		GOTO	$-1

		MOVWF	EEDAT			; Data set
		INCF	EEADR,F			; Next write location
		BSF		EECON1,WREN		; set write mode
		BCF		INTCON,GIE		; Inttrupt disable
		MOVLW	H'55'
		MOVWF	EECON2
		MOVLW	H'AA'
		MOVWF	EECON2
		BSF		EECON1,WR		; Strat write
		BSF		INTCON,GIE		; Inttrupt enable

		BANKSEL	USERWORK
		RETURN

		

		END
